//
// Created by martin on 12.11.21.
//

#ifndef ANIMTESTBED_BLENDNODE_H
#define ANIMTESTBED_BLENDNODE_H

#include "../AnimNode.h"

struct BlendNode : public AnimNode {
  BlendNode(AnimationController* animation_controller);

  virtual ~BlendNode() {}

  AnimNode* m_input_A;
  AnimNode* m_input_B;
  float m_weight;
  bool m_sync_inputs;

  ozz::vector<ozz::math::SoaTransform> m_local_matrices_A;
  ozz::vector<ozz::math::SoaTransform> m_local_matrices_B;

  virtual void Reset() {
    m_current_time = 0.f;
  }

  virtual void UpdateIsSynced(bool is_synced) override {
    m_is_time_synced = is_synced;

    if (m_sync_inputs) {
      m_is_time_synced = true;
    }

    m_input_B->UpdateIsSynced(m_is_time_synced);
    m_input_A->UpdateIsSynced(m_is_time_synced);
  }

  virtual void UpdateSyncTrack() override {
    if (m_is_time_synced) {
      m_sync_track = SyncTrack::Blend(m_weight, m_input_A->m_sync_track, m_input_B->m_sync_track);
    } else {
      assert (false);
    }
  }

  virtual void UpdateTime(float dt) {
    if (m_is_time_synced) {
      float current_time_absolute = fmodf(m_current_time * m_sync_track.m_duration + dt, 1.0f);
      float current_sync_time = m_sync_track.CalcSyncFromAbsTime(current_time_absolute);
      m_current_time = m_sync_track.CalcRatioFromSyncTime(current_sync_time);

      float time_absolute_A = fmodf (m_input_A->m_current_time * m_input_A->m_sync_track.m_duration, 1.0f);
      float sync_time_A = m_input_A->m_sync_track.CalcSyncFromAbsTime(time_absolute_A);
      float m_ratio_A = m_input_A->m_sync_track.CalcSyncFromAbsTime(sync_time_A);
      
      float time_absolute_B = fmodf (m_input_B->m_current_time * m_input_B->m_sync_track.m_duration, 1.0f);
      float sync_time_B = m_input_B->m_sync_track.CalcSyncFromAbsTime(time_absolute_B);
      float m_ratio_B = m_input_B->m_sync_track.CalcSyncFromAbsTime(sync_time_B);
      
      m_input_A->UpdateTime(current_sync_time);
      m_input_B->UpdateTime(current_sync_time);
    } else {
      m_current_time += dt;
      m_input_A->UpdateTime(dt);
      m_input_B->UpdateTime(dt);
    }
  }

  virtual void Evaluate(
      ozz::vector<ozz::math::SoaTransform>* local_matrices) override;

  virtual void GetInputNodes(std::vector<AnimNode*>& input_nodes) const override {
    input_nodes.push_back(m_input_A);
    input_nodes.push_back(m_input_B);
  };

  virtual void DrawDebugUi();
};

#endif  //ANIMTESTBED_BLENDNODE_H
